{{ log.debug() }}
<dl>
  <dt>Parameters:</dt>
  {% for parameter in parameters %}
    <dd>{{ ("**" + parameter.name + ":** " + ("`" + parameter.annotation + "` – " if parameter.annotation else "") + parameter.description)|convert_markdown(heading_level, html_id) }}</dd>
  {% endfor %}
</dl>
