{{ log.debug() }}
<p><strong>Yields:</strong></p>
<table>
  <thead>
    <tr>
      <th>Type</th>
      <th>Description</th>
    </tr>
  </thead>
  <tbody>
    <tr>
      <td>{% if yield.annotation %}<code>{{ yield.annotation }}</code>{% endif %}</td>
      <td>{{ yield.description|convert_markdown(heading_level, html_id) }}</td>
    </tr>
  </tbody>
</table>
