{{ log.debug() }}
{% if docstring_sections %}
  {% for section in docstring_sections %}
    {% if section.type == "markdown" %}
      {{ section.value|convert_markdown(heading_level, html_id) }}
    {% elif section.type == "attributes" %}
      {% with attributes = section.value %}
        {% include "attributes.html" with context %}
      {% endwith %}
    {% elif section.type == "parameters" %}
      {% with parameters = section.value %}
        {% include "parameters.html" with context %}
      {% endwith %}
    {% elif section.type == "keyword_args" %}
      {% with kwargs = section.value %}
        {% include "keyword_args.html" with context %}
      {% endwith %}
    {% elif section.type == "exceptions" %}
      {% with exceptions = section.value %}
        {% include "exceptions.html" with context %}
      {% endwith %}
    {% elif section.type == "yield" %}
      {% with yield = section.value %}
        {% include "yield.html" with context %}
      {% endwith %}
    {% elif section.type == "return" %}
      {% with return = section.value %}
        {% include "return.html" with context %}
      {% endwith %}
    {% elif section.type == "examples" %}
      {% with examples = section.value %}
        {% include "examples.html" with context %}
      {% endwith %}
    {% endif %}
  {% endfor %}
{% endif %}
