{{ log.debug() }}
<table class="field-list">
  <colgroup>
    <col class="field-name" />
    <col class="field-body" />
  </colgroup>
  <tbody valign="top">
    <tr class="field">
    <th class="field-name">Yields:</th>
    <td class="field-body">
        <ul class="first simple">
          <li>{{ ((("`" + yield.annotation + "` – ") if yield.annotation else "") + yield.description)|convert_markdown(heading_level, html_id) }}</li>
        </ul>
    </td>
    </tr>
  </tbody>
</table>
