{{ log.debug() }}
<p><strong>Parameters:</strong></p>
<table>
  <thead>
    <tr>
      <th>Name</th>
      <th>Type</th>
      <th>Description</th>
      <th>Default</th>
    </tr>
  </thead>
  <tbody>
    {% for parameter in parameters %}
      <tr>
        <td><code>{{ parameter.name }}</code></td>
        <td>{% if parameter.annotation %}<code>{{ parameter.annotation }}</code>{% endif %}</td>
        <td>{{ parameter.description|convert_markdown(heading_level, html_id) }}</td>
        <td>{% if parameter.default %}<code>{{ parameter.default }}</code>{% else %}<em>required</em>{% endif %}</td>
      </tr>
    {% endfor %}
  </tbody>
</table>
